/**
 *
 * BubbleBot Control Code
 *
 * By: Zvika Markfeld, Jonatan Bien, Uri Kareev
 *
 **/
#include <Servo.h> 
#include "HBridge.h"

// scan limit switch pin
const int switchPin = 2;
// arms spread servo pin
const int servoPin = 3;
// fan source pin
const int fanPin =  4;

// H-Bridge control pins
const int positive0Pin = 5;
const int positive1Pin = 6;
const int negative0Pin = 7;
const int negative1Pin = 8;

// Servo boundry values
const int min_pos = 85;
const int max_pos = 127;

const int NUM_OF_SPREADS = 2;
const int SPREAD_SPEED = 30;
const int UNSPREAD_SPEED = 20;
const int SPREAD_DURATION = 3000;
const int SPREAD_DURATION_DECREASE_FACTOR = 3;

// H-Bridge controller (based on http://www.mcmanis.com/chuck/robotics/tutorial/h-bridge/bjt-circuit.html, but not as nice)
HBridge hbridge;
Servo myservo;

boolean forward = true;

void setup() { 
  myservo.attach(servoPin);
  myservo.write(max_pos);
  hbridge.init(positive0Pin, positive1Pin, negative0Pin, negative1Pin);
  Serial.begin(115200);
  Serial.println("setting FORWARD for the first time");
  hbridge.forward();
}


void loop() { 

  // detect arms up
  if(digitalRead(switchPin) == HIGH) {
    Serial.println("SWITCH Detected");
    hbridge.idle();

    // switch fan on
    digitalWrite(fanPin, HIGH);

    int pos;
    int times;
    for(times = 0; times < NUM_OF_SPREADS; ++times) {
      // spread arms
      for(pos = max_pos; pos>=min_pos; pos-=1) {                                
        myservo.write(pos);
        delay(SPREAD_SPEED);
      }
      int currDelay = SPREAD_DURATION * (SPREAD_DURATION_DECREASE_FACTOR - times) / SPREAD_DURATION_DECREASE_FACTOR;
      Serial.print("Delay: "); Serial.println(currDelay); 
      delay(currDelay);
      // close arms  
      for(pos = min_pos; pos <max_pos; pos += 1)  {
        myservo.write(pos);
        delay(UNSPREAD_SPEED);
      }
    }

    // switch off fan
    digitalWrite(fanPin, LOW);

    // start bringing arms down
    if(forward) {
      Serial.println("setting BACKWARD");
      hbridge.backward();
      forward = false;
    } 
    else {
      Serial.println("setting FORWARD");
      hbridge.forward();
      forward = true;
    }
    /// wait until arm goes down and switch is off
    while (digitalRead(switchPin) == HIGH);
  }
} 

